/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland.       */
/* A CanvasGame that implements collision detection.                       */


let pieces = [];       /* These two variables are used in both this class and in the JigsawPiece class */
let currentPiece = 0;  /* Therefore, they need to be global to both classes.                           */

class JigsawCanvasGame extends CanvasGame
{
    constructor(wordList, jigsawPieceImage, pieceSize, wordY)
    {
        super();
        
        this.wordList = wordList;
        this.jigsawPieceImage = jigsawPieceImage;
        this.pieceSize = pieceSize;        
        this.wordY = wordY;

        this.currentWord = 0;
        this.createNewWordJigsaw();
    }

    createNewWordJigsaw()
    {
        currentPiece = 0;

        let colours = [[255, 0, 0], [0, 255, 0], [0, 0, 255], [255, 0, 255], [255,255,0], [255, 100, 0]];
        let currentColour = -1;  // the fixed jigsaw start piece is not contained in the colours array 
        let newColour = null; // newColour is randomly assigned below
        
        for (let letter = 0; letter < this.wordList[this.currentWord].length; letter++)
        {
            do
            {
                newColour = Math.floor(Math.random() * colours.length);
            } while (newColour === currentColour); // make sure that pieces beside each other are different colours
            currentColour = newColour;
            pieces[letter] = new JigsawPiece(this.jigsawPieceImage, colours[newColour], this.wordList[this.currentWord][letter], letter, this.pieceSize, 10, Math.random() * (canvas.width - this.pieceSize), Math.random() * (canvas.height - this.pieceSize), (this.pieceSize * 0.8) * (letter + 1), this.wordY);
        }
        this.currentWord++;
    }

    render()
    {
        super.render();
        for (let i = 0; i < pieces.length; i++)
        {
            pieces[i].render();
        }
    }

    collisionDetection()
    {
        if ((pieces.length === 0) || (this.wordList.length === 0))
        {
            return;
        }

        if (currentPiece === pieces.length)
        {
            if (this.currentWord < this.wordList.length)
            {
                gameObjects[BUTTON].start(); /* render "Continue" message */
            }
            else
            {
                for (let i = 0; i < pieces.length; i++) /* stop all gameObjects from animating */
                {
                    pieces[i].stop();
                }

                gameObjects[MESSAGE].start(); /* render "Well Done!" message */
            }
        }
    }
}