/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland.             */
/* There should always be a javaScript file with the same name as the html file. */
/* This file always holds the playGame function().                               */
/* It also holds game specific code, which will be different for each game       */





/******************** Declare game specific global data and functions *****************/
/* images must be declared as global, so that they will load before the game starts  */
let background = new Image();
background.src = "images/sunshine.png";

let jigsawPieceImage = new Image();
jigsawPieceImage.src = "images/jigsaw_piece.png";

const BACKGROUND = 0;
const ANCHOR_PIECE = 1;
const BUTTON = 2;
const MESSAGE = 3;

const JIGSAW_Y = 200;  // the y-position of the jigsaw on the canvas
const JIGSAW_PIECE_SIZE = 100; // width and height of each jigsaw piece

/******************* END OF Declare game specific data and functions *****************/







/* Always have a playGame() function                                     */
/* However, the content of this function will be different for each game */
function playGame()
{
    /* We need to initialise the game objects outside of the Game class */
    /* This function does this initialisation.                          */
    /* This function will:                                              */
    /* 1. create the various game game gameObjects                   */
    /* 2. store the game gameObjects in an array                     */
    /* 3. create a new Game to display the game gameObjects          */
    /* 4. start the Game                                                */


    /* Create the various gameObjects for this game. */
    /* This is game specific code. It will be different for each game, as each game will have it own gameObjects */
    gameObjects[BACKGROUND] = new StaticImage(background, 0, 0, canvas.width, canvas.height);
    gameObjects[ANCHOR_PIECE] = new StaticImage(jigsawPieceImage, 0, JIGSAW_Y, JIGSAW_PIECE_SIZE, JIGSAW_PIECE_SIZE);
    gameObjects[BUTTON] = new Button(BUTTON_CENTRE, 400, TEXT_WIDTH, TEXT_HEIGHT, "Continue");
    gameObjects[MESSAGE] = new StaticText("Well Done!", STATIC_TEXT_CENTRE, 450, "Times Roman", 100, "black");
    
    let selectedPiece = 0; // default to any piece initially being selected
    
    /* END OF game specific code. */

    /* Always create a game that uses the gameObject array */
    let game = new JigsawCanvasGame(["one", "two"], jigsawPieceImage, JIGSAW_PIECE_SIZE, JIGSAW_Y);

    /* Always play the game */
    game.start();

    /* Hide the "Continue" button and win message until they are needed */
    gameObjects[BUTTON].stopAndHide();
    gameObjects[MESSAGE].stopAndHide();

    /* If they are needed, then include any game-specific mouse and keyboard listners */
    document.getElementById("gameCanvas").addEventListener("mousedown", function (e)
    {
        if (e.which === 1)  // left mouse button
        {
            let canvasBoundingRectangle = document.getElementById("gameCanvas").getBoundingClientRect();
            let mouseX = e.clientX - canvasBoundingRectangle.left;
            let mouseY = e.clientY - canvasBoundingRectangle.top;

            if (gameObjects[BUTTON].pointIsInsideBoundingRectangle(mouseX, mouseY))
            {
                gameObjects[BUTTON].stopAndHide();
                game.createNewWordJigsaw();
            }
            for (let i = pieces.length - 1; i >= 0; i--)
            {
                if (pieces[i].pointIsInsideBoundingRectangle(mouseX, mouseY))
                {
                    pieces[i].setOffsetX(mouseX);
                    pieces[i].setOffsetY(mouseY);
                    selectedPiece = i;
                    break;
                }
            }
        }
    });

    document.getElementById("gameCanvas").addEventListener("mousemove", function (e)
    {
        if (e.which === 1)  // left mouse button
        {
            let canvasBoundingRectangle = document.getElementById("gameCanvas").getBoundingClientRect();
            let mouseX = e.clientX - canvasBoundingRectangle.left;
            let mouseY = e.clientY - canvasBoundingRectangle.top;

            for (let i = 0; i < pieces.length; i++)
            {
                if (selectedPiece === i)
                {
                    if (pieces[i].pointIsInsideBoundingRectangle(mouseX, mouseY))
                    {
                        pieces[i].setX(mouseX);
                        pieces[i].setY(mouseY);
                        break;
                    }
                }
            }
        }
        else if (e.which === 0) // no button selected
        {
            let canvasBoundingRectangle = document.getElementById("gameCanvas").getBoundingClientRect();
            let mouseX = e.clientX - canvasBoundingRectangle.left;
            let mouseY = e.clientY - canvasBoundingRectangle.top;
            gameObjects[BUTTON].pointIsInsideBoundingRectangle(mouseX, mouseY);
        }
    });
}